// $Id$

/**
 * Manipulation of content remove buttons.
 *
 * TableDrag objects for multiple value fields (and multigroups) are scanned
 * to find 'remove' checkboxes. These checkboxes are hidden when javascript is
 * enabled (using the Global CSS Killswitch, html.js, defined in drupal.js).
 * A new 'remove' button is created here in place of these checkboxes aimed to
 * provide a more user-friendly method to remove items. The onClick handler of
 * these buttons fire the onRemove event of the corresponding checkbox that can
 * be used by field widgets to perform additional processing.
 *
 * Custom onRemove handlers are attached to 'remove' checkboxes as follows:
 *
 * <code>
 *   $('.content-multiple-remove-checkbox').not('.my-widget-onremove-processed').each(function() {
 *     // Make sure the element is not processed more than once.
 *     $(this).addClass('my-widget-onremove-processed');
 *
 *     // Attach custom onRemove handler to each checkbox.
 *     $(this).bind('remove', function(event, settings) {
 *       if (settings.type == 'my_widget_type') {
 *         // Custom code to process the remove event of the widget.
 *       }
 *     });
 *   });
 * </code>
 *
 * Note that onRemove handlers should filter by settings.type and/or
 * settings.name before processing the event.
 *
 * onRemove handlers are invoked with the following arguments:
 *   event     - The onClick event object.
 *   settings  - An object with the following properties:
 *     tableDrag    - The tableDrag object.
 *     checkbox_id  - The ID of the checkbox element.
 *     checkbox     - The DOM object of the remove checkbox element.
 *     button       - The DOM object of the remove button element.
 *     type         - field or multigroup.
 *     name         - The name of the field or multigroup.
 *     delta        - The delta of the field or fields in the subgroup.
 */
Drupal.behaviors.contentRemoveButtons = function(context) {
  $('table.content-multiple-table', context).not('.content-multiple-remove-buttons-processed').each(function() {
    var table = this, tableDrag = Drupal.tableDrag[$(table).attr('id')];

    // Make sure the element is not processed more than once.
    $(table).addClass('content-multiple-remove-buttons-processed');

    // Create remove buttons, only when there is more than one item.
    if ($('input.content-multiple-remove-checkbox', table).length > 1) {
      $('input.content-multiple-remove-checkbox', table).each(function() {
        Drupal.contentRemoveButtons.createButton(tableDrag, this);
      });
    }
  });
};

/**
 * Global contentRemoveButtons object.
 */
Drupal.contentRemoveButtons = {
  _isBusy: false
};

/**
 * Create a 'remove' button.
 */
Drupal.contentRemoveButtons.createButton = function(tableDrag, checkbox) {
  // Parse checkbox options passed via alt attribute.
  var settings = {}, checkbox_alt = $(checkbox).attr('alt');
  if (typeof checkbox_alt != 'string' || checkbox_alt.length <= 0) {
    return;
  }
  checkbox_alt = checkbox_alt.split(',');
  for (var i = 0; i < checkbox_alt.length; i++) {
    var option = checkbox_alt[i].split(':');
    if (option.length == 2) {
      settings[option[0]] = option[1];
    }
  }
  // Ignore checkbox if required settings are not present.
  if (settings.type == undefined || settings.name == undefined || settings.delta == undefined) {
    return;
  }

 // Create a new remove button.
  var button = $(Drupal.theme('contentRemoveButton', settings.type, settings.name, settings.delta));

  // Complete the button settings.
  $.extend(settings, {
    tableDrag: tableDrag,
    checkbox_id: $(checkbox).attr('id'),
    checkbox: checkbox,
    button: button.get(0)
  });

  // Bind the onClick event to the 'remove' button.
  button.bind('click', function(event) {
    $(this).blur();

    // Do not allow users click on a 'remove' button while the process of
    // another one has not been finished yet.
    if (!Drupal.contentRemoveButtons._isBusy) {
      Drupal.contentRemoveButtons._isBusy = true;
      Drupal.contentRemoveButtons.onClick(event, settings);
    }

    return false;
  });

  // Attach the new button to the DOM tree.
  $(checkbox).parent().append(button);
};

/**
 * onClick handler for 'remove' buttons.
 */
Drupal.contentRemoveButtons.onClick = function(event, settings) {
  // Display the throbber while remove callbacks are being processed.
  $(settings.button).addClass('content-multiple-throbber');

  // Check the checkbox related to the 'remove' button.
  $('input[@id='+ settings.checkbox_id +']').attr('checked', 'checked');

  // Trigger all onRemove event handlers attached to the checkbox.
  $(settings.checkbox).trigger('remove', [event, settings]);

  // Hide the table row.
  $(settings.button).parents('tr:first').fadeOut('slow', function() {
    $(this).addClass('content-multiple-removed-row');

    // When there's only one delta, we don't want to remove any more items, so
    // we hide the table column where the remove checkboxes and buttons are.
    if ($('input.content-multiple-remove-checkbox', settings.tableDrag.table).not(':checked').length <= 1) {
      $('th.content-multiple-remove-header', settings.tableDrag.table).fadeOut();
      $('td.content-multiple-remove-cell', settings.tableDrag.table).fadeOut();
    }

    // Restripe table rows and display table changed warning.
    settings.tableDrag.restripeTable();
    settings.tableDrag.displayChangedWarning();

    // Disable the busy flag after a while.
    setTimeout(function() { Drupal.contentRemoveButtons._isBusy = false; }, 500);
  });

  // Hide the throbber after all remove callbacks have been processed.
  $(settings.button).removeClass('content-multiple-throbber');
};

/**
 * tableDrag override; Re-assign odd/even classes to all visible table rows.
 */
Drupal.tableDrag.prototype.restripeTable = function() {
  var odd = true;
  $('tr.draggable:visible', this.table).each(function() {
    if (odd) {
      $(this).filter('.even').removeClass('even').addClass('odd');
    }
    else {
      $(this).filter('.odd').removeClass('odd').addClass('even');
    }
    odd = !odd;
  });
};

/**
 * tableDrag extension; Display table change warning when appropriate.
 */
Drupal.tableDrag.prototype.displayChangedWarning = function() {
  if (this.changed == false) {
    $(Drupal.theme('tableDragChangedWarning')).insertAfter(this.table).hide().fadeIn('slow');
    this.changed = true;
  }
};

/**
 * Theme the 'remove' button.
 *
 * @param type
 *   'field' or 'multigroup'.
 * @param name
 *   The name of the field or multigroup.
 * @param delta
 *   The delta of the field or fields in the subgroup.
 */
Drupal.theme.prototype.contentRemoveButton = function(type, name, delta) {
  return '<a href="javascript:void(0)" class="content-multiple-remove-button" title="'+ Drupal.t('Remove this item') +'"></a>';
};